/*******************************************************************************
                        List Utiltity Predicates
*******************************************************************************/

/******** Checking for membership in a list.
   member_op(I,L)
Item I is a member of the list L.
********/

   member_op(X,[X|_]).
   member_op(X,[_|Z]) :- member_op(X,Z) .

/********  Appending lists to lists.
   append_op(L1,L2,R)
R is the result of appending list L2 to the end of list L1.
*/
   append_op([],L,L).
   append_op([X|L1],L2,[X|L3]) :- append_op(L1,L2,L3) .

/******** Deleting an object from a list
   delete_op(I,L,R)
R is the result of deleting item I from the list L.
********/

   delete_op(X,[X|Y],Y).
   delete_op(X,[Y|W],[Y|Z]) :- delete_op(X,W,Z).

/******** Determine the prefix of a list.
   prefix_op(P, L)   P is a prefix of list L
   Note [] is a prefix!
********/
/* 2 definitions */

pa(P,L) :- append_op(P,_,L).

prefix([],_).
prefix([A|B],[A|C]) :- prefix(B,C).

/******** Determine the suffix of a list.
   suffix_op(S, L)   S is a suffix of list L
   Note [] is a suffix!
********/
/* 2 definitions */

sa(S,L) :- append_op(_,S,L).

suffix(L,L).
suffix(S,[_|L]) :- suffix(S,L).

/******** Determine a sublist of a list.
   sublist_op(S, L)   S is a sublist of list L
********/
/* 2 definitions */

sla(S,L) :- append_op( _,S,Lt) ,
            append_op(Lt,_,L).

sublist(S,L) :- prefix(S,L).
sublist(S,[_|L]) :- sublist(S,L).
