/****************
 Define has, isa and flies as operators to
make Prolog look more English like */

:- op( 100, xfx, [ has , isa , flies ]).

/* Create some rules and facts using the operators */

Animal has hair  :-  Animal isa mammal.
Animal has feathers  :-  Animal isa bird.

owl isa bird.
cat isa mammal.
dog isa mammal.

/* Example queries are the following.

A isa bird.
B isa mammal.

owl isa What.
owl has feathers.
*/

/****************
Play sports examples
*/
:- op(300, xfx, plays).
:- op(200, xfy, and).

/* Example queries

Term1 = jimmy plays football and squash.
Term2 = susan plays tennis and basketball and volleyball.

Recall that everything within Prolog consists of compound terms.

Term1 = plays(jimmy, and(football, squash)).
Term2 = plays(susan,
              and(tennis,
                  and(basketball, volleyball))).

Term1 = plays(jimmy, and(football, squash)) ,  Term1 =..[F,C1,C2].

Term2 = susan plays tennis and basketball and volleyball ,
  Term2 =.. [F1,C11,C12] , C12 =.. [F2,C21,C22], C22 =.. [F3,C31,C32].

*/

/*************************************
 Example for one of DeMorgan's laws
 *************************************/

/* Define some logical operators */

:-  op( 800, xfx,  <==> ).
:-  op( 700, xfy, v ).
:-  op( 600, xfy, & ).
:-  op( 500, fy, ~ ).

/* Define one of De Morgan's laws  using infix form of the rule. */

~(A & B) <==> ~A v ~B
    :- write('Infix form of rule').

/* Define the same law using compound terms. 

equivalence( not( and( A, B ) ),
             or( not( A ), not( B ) ) ).
*/

<==>( ~( &( A, B ) ),
        v( ~( A ), ~( B ) ) )
       :- write('Prefix form of rule').

/* Example queries.  Notice that using
;<return> both rules are used. 

~(a & b) <==> X v ~b.
~(a & b) <==> X v Y.
<==>(~(&(A,B) ),v(~(A),~(B))).
*/

:- op(100, xfy, op1).
:- op(200, yfx, op2).
:- op(300, xfx, op3).

/* Example queries for the operators op1, op2 & op3.  The value of L
shows the internal representation for the clauses.

C = 1 op1 2 op1 3 op1 4 , C =.. L.

C = 1 op2 2 op2 3 op2 4 , C =.. L.

C = 1 op3 2 op3 3 op3 4 , C =.. L.

C = 1 op1 2 op3 3 op2 4 , C =.. L.
*/
