:- [library(clpfd)].

/************************************
Cryptarithmetic puzzle SEND + MORE = MONEY

A set of six algorithms.
Query:  stats_1(Time) ... stats_6(Time) to get timing for each of the versions.
************************************/
% Standard Prolog solution
% Figure 5.8, p100, Thinking as Computation, Hector J. Levesque

stats_1(Time) :-
  statistics(runtime,_),
  soln_1([S,E,N,D]+[M,O,R,E]=[M,O,N,E,Y]),
  write_ans([S,E,N,D]+[M,O,R,E]=[M,O,N,E,Y]),
  statistics(runtime,[_,Time]).

soln_1([S,E,N,D]+[M,O,R,E]=[M,O,N,E,Y]) :-
  uniq_digits_1(S,E,N,D,M,O,R,Y),
  S > 0, M > 0,
  Y is (D + E) mod 10, C1 is (D + E) //10,
  E is (N + R + C1) mod 10, C10 is (N + R + C1) // 10,
  N is (E + O + C10) mod 10, C100 is (E + O + C10) // 10,
  O is (S + M + C100) mod 10, M is (S + M + C100) // 10.

uniq_digits_1(S,E,N,D,M,O,R,Y) :-
  dig(S), dig(E), dig(N), dig(D), dig(M), dig(O), dig(R), dig(Y),
  S=\=E, S=\=N, S=\=D, S=\=M, S=\=O, S=\=R, S=\=Y,
  E=\=N, E=\=D, E=\=M, E=\=O, E=\=R, E=\=Y,
  N=\=D, N=\=M, N=\=O, N=\=R, N=\=Y,
  D=\=M, D=\=O, D=\=R, D=\=Y,
  M=\=O, M=\=R, M=\=Y,
  O=\=R, O=\=Y,
  R=\=Y.

dig(0). dig(1). dig(2). dig(3). dig(4).
dig(5). dig(6). dig(7). dig(8). dig(9).

write_ans([S,E,N,D]+[M,O,R,E]=[M,O,N,E,Y]) :-
  write('  '), write(S), write(E), write(N), write(D), nl,
  write('  '), write(M), write(O), write(R), write(E), nl,
  write(' '),  write(M), write(O), write(N), write(E), write(Y).

%-------------------------------------------------------------------------------

stats_2(Time) :-
  statistics(runtime,_),
  soln_2([S,E,N,D]+[M,O,R,E]=[M,O,N,E,Y]),
  write_ans([S,E,N,D]+[M,O,R,E]=[M,O,N,E,Y]),
  statistics(runtime,[_,Time]).

soln_2([S,E,N,D]+[M,O,R,E]=[M,O,N,E,Y]) :- 
  uniq_digits_2(S,E,N,D,M,O,R,Y),
  S > 0, M > 0,
  Y is (D + E) mod 10, C1 is (D + E) //10,
  E is (N + R + C1) mod 10, C10 is (N + R + C1) // 10,
  N is (E + O + C10) mod 10, C100 is (E + O + C10) // 10,
  O is (S + M + C100) mod 10, M is (S + M + C100) // 10.

uniq_digits_2(S,E,N,D,M,O,R,Y) :-
  dig(S), dig(E),
  S=\=E, dig(N),
  S=\=N, E=\=N, dig(D),
  S=\=D, E=\=D, N=\=D, dig(M),
  S=\=M, E=\=M, N=\=M, D=\=M, dig(O),
  S=\=O, E=\=O, N=\=O, D=\=O, M=\=O, dig(R),
  S=\=R, E=\=R, N=\=R, D=\=R, M=\=R, O=\=R, dig(Y),
  S=\=Y, E=\=Y, N=\=Y, D=\=Y, M=\=Y, O=\=Y, R=\=Y.

%-------------------------------------------------------------------------------

stats_3(Time) :-
  statistics(runtime,_),
  soln_3([S,E,N,D]+[M,O,R,E]=[M,O,N,E,Y]),
  write_ans([S,E,N,D]+[M,O,R,E]=[M,O,N,E,Y]),
  statistics(runtime,[_,Time]).

soln_3([S,E,N,D]+[M,O,R,E]=[M,O,N,E,Y]) :- 
  uniq_digits_2(S,E,N,D,M,O,R,Y),
  S > 0, M > 0,
  LS is 1000*S + 100*E + 10*N + D + 1000*M + 100*O + 10*R + E,
  RS is 10000*M + 1000*O + 100*N + 10*E + Y,
  RS = LS.

%-------------------------------------------------------------------------------

stats_4(Time) :-
  statistics(runtime,_),
  soln_4([S,E,N,D]+[M,O,R,E]=[M,O,N,E,Y]),
  write_ans([S,E,N,D]+[M,O,R,E]=[M,O,N,E,Y]),
  statistics(runtime,[_,Time]).

soln_4([S,E,N,D]+[M,O,R,E]=[M,O,N,E,Y]) :- 
  assign([0,1,2,3,4,5,6,7,8,9], [S,E,N,D,M,O,R,Y]),
  S > 0, M > 0,
  LS is 1000*S + 100*E + 10*N + D + 1000*M + 100*O + 10*R + E,
  RS is 10000*M + 1000*O + 100*N + 10*E + Y,
  RS = LS.

% assign( Digits, Vars):  Assign chosen distinct digits from list Digits to
% variables in list Vars

assign( _, [ ]).		% No variable to be assigned a digit

assign( Digs, [D | Vars])  :-	
  mydelete( D, Digs, Digs1),		% Deleting D from list Digs gives Digs1
  assign( Digs1, Vars).		% Assign digits to remaining variables

% Delete from list defined as in the class slides.
% mysdelete(Item, List, Result) asserts that Result is List with the
% first instance of Item at the top level removed from the List.

mydelete( X, [X | L], L).

mydelete( X, [Y | L], [Y | L2]) :- mydelete( X, L, L2).

%-------------------------------------------------------------------------------

stats_5(Time) :-
  statistics(runtime,_),
  soln_5([S,E,N,D]+[M,O,R,E]=[M,O,N,E,Y]),
  write_ans([S,E,N,D]+[M,O,R,E]=[M,O,N,E,Y]),
  statistics(runtime,[_,Time]).

soln_5([S,E,N,D]+[M,O,R,E]=[M,O,N,E,Y]) :-
  dig(D), dig(E),
  Y is (D + E) mod 10, C1 is (D + E) //10,
  dig(N), dig(R),
  E is (N + R + C1) mod 10, C10 is (N + R + C1) // 10,
  dig(O),
  N is (E + O + C10) mod 10, C100 is (E + O + C10) // 10,
  dig(S), S > 0, dig(M), M > 0,
  O is (S + M + C100) mod 10, M is (S + M + C100) // 10,
  uniq_digits_2(S,E,N,D,M,O,R,Y).



stats_6(Time) :-
  statistics(runtime,_),
  soln_6([S,E,N,D]+[M,O,R,E]=[M,O,N,E,Y]),
  write_ans([S,E,N,D]+[M,O,R,E]=[M,O,N,E,Y]),
  statistics(runtime,[_,Time]).

soln_6([S,E,N,D]+[M,O,R,E]=[M,O,N,E,Y]) :-
  Vars = [S,E,N,D,M,O,R,Y],                % All variables in the puzzle
  Vars ins 0..9,                           % They are all decimal digits
  all_different(Vars),                     % They are all different
  1000*S + 100*E + 10*N + D  +
  1000*M + 100*O + 10*R + E  #=
  10000*M + 1000*O + 100*N + 10*E + Y ,
  M #\= 0 , S #\= 0 ,

/* Assign a value to each variable in Vars; i.e. systematically try out values
for the finite domain variables in the set Vars until all of them are ground.
*/
  labeling( [], Vars).
