%  Prolog programs to solve the zebra problem.

/*
0. There are:
- five houses, each of a different colour
- five people, each of a different nationality
- five different pets
- five different cigarette brands
- five different drinks.

1. The Englishman lives in the red house.
2. The Spaniard owns the dog.
3. Coffee is drunk in the green house.
4. The Ukrainian drinks tea.
5. The green house is immediately to the right (your right) of the ivory house.
6. The Old Gold smoker owns snails.
7. Kools are smoked in the yellow house.
8. Milk is drunk in the middle house.
9. The Norwegian lives in the first house.
10. The man who smokes Chesterfields lives in the house next to the man with the fox.
11. Kools are smoked in the house next to the house where the horse is kept.
12. The Lucky Strike smoker drinks orange juice.
13. The Japanese smokes Parliaments.
14. The Norwegian lives next to the blue house.

Now, who drinks water? Who owns the zebra?
*******************************************************************************/
% Thinking as Computation, Hector J. Levesque, p111, Fig 5.17
%
% The encoding is the position 1..5 of the house in which the variable is
% true.  Modified to use operators for right_of and next_to.

% Operators

:- op(500, xfx, [right_of, next_to]).

%  The facts.

find_2(ZebraOwner, WaterDrinker) :-  % Only gives house number not person
  England = Red,              %  1
  Spain = Dog,                %  2
  Coffee = Green,             %  3
  Ukraine = Tea,              %  4
  Green right_of Ivory,       %  5
  Winston = Snail,            %  6
  Kool = Yellow,              %  7
  middle(Milk),               %  8
  leftmost(Norway),           %  9
  Chesterfield next_to Fox,   % 10
  Kool next_to Horse,         % 11
  LuckyStrike = OJ,           % 12
  Japan = Parliament,         % 13
  Norway next_to Blue,        % 14
  
  unique_pos(Green, Red, Yellow, Ivory, Blue),
  unique_pos(England, Spain, Ukraine, Japan, Norway),
  unique_pos(Dog, Snail, ZebraOwner, Fox, Horse),
  unique_pos(Tea, Milk, OJ, Coffee, WaterDrinker),
  unique_pos(Winston, Kool, Parliament, Chesterfield, LuckyStrike).

%***********************  Support routines *******************

unique_pos(P1, P2, P3, P4, P5) :-
  pos(P1), pos(P2), pos(P3), pos(P4), pos(P5),
  unique([P1, P2, P3, P4, P5]).

pos(1). pos(2). pos(3). pos(4). pos(5).

unique([]).
unique([F | R]) :- \+ member(F,R) , unique(R).

leftmost(1).
middle(3).
2 right_of 1. 3 right_of 2. 4 right_of 3. 5 right_of 4.

X next_to Y :- X right_of Y ; Y right_of X.

%*******************************************************

soln_3(ZebraOwner, WaterDrinker, England, Spain, Ukraine, Japan, Norway) :-  
  England = Red,              %  1
  Spain = Dog,                %  2
  Coffee = Green,             %  3
  Ukraine = Tea,              %  4
  Green right_of Ivory,       %  5
  Winston = Snail,            %  6
  Kool = Yellow,              %  7
  middle(Milk),               %  8
  leftmost(Norway),           %  9
  Chesterfield next_to Fox,   % 10
  Kool next_to Horse,         % 11
  LuckyStrike = OJ,           % 12
  Japan = Parliament,         % 13
  Norway next_to Blue,        % 14

  unique_pos(Green, Red, Yellow, Ivory, Blue),
  unique_pos(England, Spain, Ukraine, Japan, Norway),
  unique_pos(Dog, Snail, ZebraOwner, Fox, Horse),
  unique_pos(Tea, Milk, OJ, Coffee, WaterDrinker),
  unique_pos(Winston, Kool, Parliament, Chesterfield, LuckyStrike).

find_3(ZebraOwner, WaterDrinker) :-
  soln_3(ZebraOwner, WaterDrinker, E, S, U, J, N),
  pos_match_1(ZebraOwner, E, englishman),
  pos_match_1(ZebraOwner, S, spaniard),
  pos_match_1(ZebraOwner, U, ukranian),
  pos_match_1(ZebraOwner, J, japanese),
  pos_match_1(ZebraOwner, N, norwegian),
  pos_match_2(WaterDrinker, E, englishman),
  pos_match_2(WaterDrinker, S, spaniard),
  pos_match_2(WaterDrinker, U, ukranian),
  pos_match_2(WaterDrinker, J, japanese),
  pos_match_2(WaterDrinker, N, norwegian).

pos_match_1(X, X, Name) :-
  write('The zebra owner is '), write(Name).

pos_match_1(X, Y, _) :- X \= Y.

pos_match_2(X, X, Name) :-
  nl, write('The water drinker is '), write(Name).

pos_match_2(X, Y, _) :- X \= Y.

/******************************************************************************/
%  Aug 1984, Dan Sahlin, The Royal Inst. of Techn., Stockholm, Sweden
% (revised October, 1985, P. H. Roosen-Runge)

	:- op(100,xfy,on).  % a bit of grammar

find_1(ZebraOwner, WaterDrinker) :-

%	************  THE FACTS **************************************
% house( Colour, Nationality, Pet, Drink, Cigarette)

	makehouses(5, List),                                              % 0
	house(  red,  englishman,    _,      _,       _)   on List,       % 1
	house(    _,    spaniard,  dog,      _,       _)   on List,       % 2
	house(green,           _,    _, coffee,       _)   on List,       % 3
	house(    _,   ukrainian,    _,    tea,       _)   on List,       % 4

    sublist( [house(ivory,           _,    _,      _,       _) ,
      house(green,           _,    _,      _,       _)], List),     % 5

	house(    _,           _,snail,      _,old_gold)   on List,       % 6
	house(yellow,          _,    _,      _,   kools)   on List,       % 7

     List = [_, _, 
	house(    _,           _,    _,   milk,       _),
	     _, _],                                                       % 8
     List = [house(    _,   norwegian,    _,      _,       _) | _], % 9

 nextto(house(    _,           _,    _,      _, chesterfields),
	house(    _,           _,  fox,      _,         _), List),        % 10

 nextto(house(    _,           _,    _,      _,    kools),
        house(    _,           _,horse,      _,        _), List),   % 11

	house(    _,           _,    _, orange, lucky_strike) on List,    % 12
	house(    _,    japanese,    _,      _, parliaments ) on List,    % 13
 
 nextto(house(    _,   norwegian,    _,      _,          _),
	house( blue,           _,    _,      _,          _), List),       % 14

	house(    _, WaterDrinker,    _,  water,          _)  on List,
	house(    _,  ZebraOwner,zebra,      _,          _)  on List.


%       ********** DEFINITIONS ************************************

	makehouses(0,[]).

	makehouses(N,[house(_, _, _, _, _)|List]) 
		:- N>0, N1 is N - 1, makehouses(N1,List).

	X on [X | _].
	X on [_ | R]  :-  X on R.

	sublist(S, L) :- add(S, _, L).
	sublist(S, [_ | T]) :- sublist(S, T).

	add([], L, L).
	add([X | R], Y, [X | T]) :- add(R,Y,T).

	nextto(H1, H2, L) :- sublist([H1, H2], L).
        nextto(H1, H2 ,L) :- sublist([H2, H1], L).
